'''
  @file   BaeLogHandler.py
  @author zhangguanxing (Gethin Zhang) <zhangguanxing01@baidu.com; gethinzhang@gmail.com>
  @date   Wed Sep  4 16:09:46 2013
  
  @brief  BAE distribute log appender
  
  
'''
import time
import traceback
import logging
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), "lib"))

from   client import Client
from   utils  import *

class BaeLogHandler(logging.Handler):
    def __init__(self, ak, sk, raise_error = False):
        logging.Handler.__init__(self)
        self._client = Client(ak, sk, raise_error)
    
    def flush(self):
        self._client.flush()

    def close(self):
        self._client.close()

    def emit(self, record):
        msg = self.format(record)
        tag = record.name
        if tag == "root":
            tag = None        

        if record.exc_info:
            record.levelno = logging.WARNING
            
        if not self._client.is_open():
            self._client.open()
        self._client.log(getbaelevel(record.levelno), msg, record.created * 1000, tag)

    def __del__(self):
        if hasattr(self, "_client"):
            self.close()


if __name__ == "__main__":
    import logging
    handler = BaeLogHandler('python', 'pythonpwd', True)
    logger = logging.getLogger("python");
    logger.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(asctime)s %(levelno)s [%(lineno)s] %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    logger.debug("debug message")

    '''
    logger.info("info message")
    logger.warning("warning message")
    logger.fatal("fatal message")
    logger.log(12, "trace message")


    try:
        1/0
    except:
        logger.exception("exception message")
    '''
