import os
import sys
import time

from thrift.transport import TTransport
from thrift.transport import TSocket
from thrift.transport import THttpClient
from thrift.protocol  import TBinaryProtocol

from logchain import logchain
from logchain.ttypes import *
from thrift import TSerialization
from constants import *

class Client:
    def __init__(self, ak, sk, raise_error = False):
        self._logs  = []
        self._raise = raise_error
        self.ak = ak
        self.sk = sk

        self._last_flush = time.time()

        if set(("BAE_ENV_LOG_HOST", "BAE_ENV_APPID", "BAE_ENV_LOG_PORT")).issubset(os.environ):
            self._host  = os.environ["BAE_ENV_LOG_HOST"]
            self._appid = os.environ["BAE_ENV_APPID"]
            port  = os.environ["BAE_ENV_LOG_PORT"]
            if not port.isdigit():
                raise TypeError("env port error")
            else:
                self._port = int(port)
        else:
            if  self._raise:
                raise NotImplementedError("please don't use bae log outside BAE 3.0 environment")

        self._socket    = TSocket.TSocket(self._host, self._port)
        self._socket.setTimeout(CLIENT_TIMEOUT)
        self._transport = TTransport.TFramedTransport(self._socket)
        self._protocol  = TBinaryProtocol.TBinaryProtocol(self._transport)
        self._client    = logchain.Client(self._protocol)
        
    def is_open(self):
        return self._transport.isOpen()

    def open(self):
        try:
            self._transport.open()
        except (TException, TTransport.TTransportException) as e:
            if self._raise:
                raise
            else:
                pass

    def close(self):
        try:
            self.flush()
        except TException:
            pass

        self.flush()
        self._transport.close()

    def flush(self):
        if not self.is_open():
            self.open()

        secret = SecretEntry(self.ak, self.sk)
        log = BaeLog(self._logs, secret)

        ret = BaeRet.RETRY

        try:
            ret = self._client.log(log)
        except TException:
            pass
            
        if ret == BaeRet.OK:
            self._logs = []
            self._last_flush = time.time()
            return

    def should_flush(self):
        if len(self._logs) >= CACHE_SHOULD_DROP:
            self._logs = []
            self._last_flush = time.time()
            return False
        if len(self._logs) >= CACHE_SHOULD_FLUSH or time.time() - self._last_flush >= FLUSH_TIMEOUT:
            return True

    def log(self, level, msg, timestamp, tag = None):
        if not self.is_open():
            if not self.open():
                return

        userlog = UserLogEntry()
        
        userlog.appid = self._appid
        userlog.level = level
        if tag and tag != "root":
            userlog.tag = tag
        userlog.timestamp = timestamp
        userlog.msg       = msg

        buf = TSerialization.serialize(userlog)
        
        baelog = BaeLogEntry()
        baelog.category = "user"
        baelog.content  = buf

        self._logs.append(baelog)
        if self.should_flush():
            self.flush()
